@@ -71,10 +71,10 @@ int main() {
     parabola[ny] = 1.0 - square(y[ny]);
     parabola2[ny] = 0.5*(1.0 - square(y[ny]));
   }
-  trans.makeSpectral(sinusoid);
-  trans.makeSpectral(parabola);
-  trans.makeSpectral(parabola2);
-  trans.makeSpectral(zero);
+  sinusoid.makeSpectral(trans);
+  parabola.makeSpectral(trans);
+  parabola2.makeSpectral(trans);
+  zero.makeSpectral(trans);
 
   FlowField un(Nx,Ny,Nz,Nd,Lx,Lz,a,b);
 
@@ -116,7 +116,6 @@ int main() {
   ofstream ts("t.asc");
   ts << "% time" << endl;
 
-  trans.makePhysical(Ubase);
   Ubase.save("Ubase");
 
   for (Real t=T0; t<T1; t += n*dt) {
