@@ -81,7 +81,7 @@ public:
   friend void swap(Vector& f, Vector& g); // swap data w/o copying arrays
 
 protected:
-  Real* data_;
+  Real* data_; // allocated and freed using FFTW routines for byte-alignment
   int N_;
 };
 
