@@ -124,7 +124,7 @@ void fftw_loadwisdom(const char* filename_) {
 
   FILE* input_file;
   input_file = fopen(filename, "r");
-  if (FFTW_FAILURE == fftw_import_wisdom_from_file(input_file)) 
+  if (!fftw_import_wisdom_from_file(input_file)) 
     cerr << "Error reading fftw-wisdom file " << filename << ", proceding without fftw-wisdom.\n";
   else
     fclose(input_file);
@@ -245,27 +245,72 @@ ChebyCoeff::~ChebyCoeff() {}
 
 void ChebyCoeff::chebyfft() {
   ChebyTransform t(N_);
-  t.chebyfft(*this);
+  chebyfft(t);
 }
 void ChebyCoeff::ichebyfft(){
   ChebyTransform t(N_);
-  t.ichebyfft(*this);
+  ichebyfft(t);
 }
 void ChebyCoeff::makeSpectral() {
   ChebyTransform t(N_);
-  t.makeSpectral(*this);
+  makeSpectral(t);
 }
 void ChebyCoeff::makePhysical() {
   ChebyTransform t(N_);
-  t.makePhysical(*this);
+  makePhysical(t);
 }
 void ChebyCoeff::makeState(fieldstate s){ 
   if (state_ != s) {       // need to change state?
     ChebyTransform t(N_);
     if (state_==Physical)  // state is Physical; change to Spectral
-      t.chebyfft(*this);
+      chebyfft(t);
     else
-      t.ichebyfft(*this);  // state is Spectral; change to Physical
+      ichebyfft(t);  // state is Spectral; change to Physical
+  }
+}
+
+void ChebyCoeff::chebyfft(const ChebyTransform& t) {
+  assert(t.N() == N_);
+  assert(state_ == Physical);
+
+  fftw_execute_r2r(t.cosfftw_plan_, data_, data_);
+
+  Real nrm = 1.0/(N_-1); // needed because FFTW does unnormalized transforms
+  data_[0] *= 0.5*nrm;
+  for (int n=1; n<N_; ++n)
+    data_[n] *= nrm;
+  
+  state_ = Spectral;
+  return;
+}
+
+void ChebyCoeff::ichebyfft(const ChebyTransform& t){
+  assert(t.N() == N_);
+  assert(state_ == Spectral);
+
+  // 0th elem is different due to reln between cheby and cos transforms.
+  data_[0] *= 2.0;
+  fftw_execute_r2r(t.cosfftw_plan_, data_, data_);
+  for (int n=0; n<N_; ++n)
+    data_[n] *= 0.5;
+  
+  state_= Physical;
+  return;
+}
+void ChebyCoeff::makeSpectral(const ChebyTransform& t) {
+  if (state_ == Physical)
+    chebyfft(t);
+}
+void ChebyCoeff::makePhysical(const ChebyTransform& t) {
+  if (state_ == Spectral)
+    ichebyfft(t);
+}
+void ChebyCoeff::makeState(fieldstate s, const ChebyTransform& t) { 
+  if (state_ != s) {       // need to change state?
+    if (state_==Physical)  // state is Physical; change to Spectral
+      chebyfft(t);
+    else
+      ichebyfft(t);  // state is Spectral; change to Physical
   }
 }
 
@@ -288,11 +333,7 @@ void ChebyCoeff::randomize(Real decay, bool a_dirichlet, bool b_dirichlet) {
   else if (a_dirichlet)
     data_[0] -= eval_b();
 
-  if (startState == Physical) {
-    ChebyTransform t(N_);
-    t.ichebyfft(*this);
-  }
-  
+  makeState(startState);
 }
 
 void ChebyCoeff::setToZero() {
@@ -475,7 +516,6 @@ void ChebyCoeff::binaryLoad(istream& is) {
   }
 }
 
-
 ChebyCoeff& ChebyCoeff::operator*=(Real c) {
   for (int i=0; i<N_; ++i)
     data_[i] *= c;
@@ -526,6 +566,7 @@ ChebyCoeff operator-(const ChebyCoeff& u, const ChebyCoeff& v) {
   rtn -= v;
   return rtn;
 }
+
 bool operator==(const ChebyCoeff& u, const ChebyCoeff& v) {
   if (!u.congruent(v))
     return false;
@@ -535,6 +576,10 @@ bool operator==(const ChebyCoeff& u, const ChebyCoeff& v) {
   return true;
 }
 
+bool operator!=(const ChebyCoeff& u, const ChebyCoeff& v) {
+  return !(u==v);
+}
+
 void swap(ChebyCoeff& f, ChebyCoeff& g) {
   Real rtmp = f.a_;
   f.a_ = g.a_;
@@ -966,29 +1011,45 @@ void ComplexChebyCoeff::resize(int N) {re.resize(N); im.resize(N);}
 
 void ComplexChebyCoeff::chebyfft() {
   ChebyTransform t(re.numModes());
-  t.chebyfft(*this);
+  chebyfft(t);
 }
 void ComplexChebyCoeff::ichebyfft(){
   ChebyTransform t(re.numModes());
-  t.ichebyfft(*this);
+  ichebyfft(t);
 }
 void ComplexChebyCoeff::makeSpectral() {
   ChebyTransform t(re.numModes());
-  t.makeSpectral(*this);
+  makeSpectral(t);
 }
 void ComplexChebyCoeff::makePhysical() {
   ChebyTransform t(re.numModes());
-  t.makePhysical(*this);
+  makePhysical(t);
 }
 void ComplexChebyCoeff::makeState(fieldstate s){ 
-  assert(re.state() == im.state());
-  if (re.state() != s) {       // need to change state
-    ChebyTransform t(re.numModes());
-    if (re.state()==Physical)  // state is Physical, change to Spectral
-      t.chebyfft(*this);
-    else
-      t.ichebyfft(*this);      // state is Spectral, change to Physical
-  }
+  ChebyTransform t(re.numModes());
+  re.makeState(s,t);
+  im.makeState(s,t);
+}
+
+void ComplexChebyCoeff::chebyfft(const ChebyTransform& t) {
+  re.chebyfft(t);
+  im.chebyfft(t);
+}
+void ComplexChebyCoeff::ichebyfft(const ChebyTransform& t) {
+  re.ichebyfft(t);
+  im.ichebyfft(t);
+}
+void ComplexChebyCoeff::makeSpectral(const ChebyTransform& t) {
+  re.makeSpectral(t);
+  im.makeSpectral(t);
+}
+void ComplexChebyCoeff::makePhysical(const ChebyTransform& t) {
+  re.makePhysical(t);
+  im.makePhysical(t);
+}
+void ComplexChebyCoeff::makeState(fieldstate s, const ChebyTransform& t) {
+  re.makeState(s,t);
+  im.makeState(s,t);
 }
 
 void ComplexChebyCoeff::randomize(Real decay, bool a_dirichlet, bool b_dirichlet) {
@@ -1123,6 +1184,9 @@ bool ComplexChebyCoeff::congruent(const ComplexChebyCoeff& v) const {
 bool operator==(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v) {
   return u.re == v.re && u.im == v.im;
 }
+bool operator!=(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v) {
+  return !(u==v);
+}
 
 void swap(ComplexChebyCoeff& f, ComplexChebyCoeff& g) {
   swap(f.re,g.re);
@@ -1200,94 +1264,59 @@ ComplexChebyCoeff integrate(const ComplexChebyCoeff& du) {
 // ====================================================================
 ChebyTransform::ChebyTransform(int N, int flags) 
   : N_(N),
-    sin_table_(new Real[N-1]),
-    cos_table_(new Real[N-1]),
-    q_(new Real[N-1]),
-    Q_(new fftw_complex[(N-1)/2 +1]),
-    flags_(flags),
-    realfft_plan_(0)
+    flags_(flags | FFTW_DESTROY_INPUT),
+    cosfftw_plan_(0)
 {
   assert(N_ >0);
-  assert(N_%2 == 1);
 
-  int Nb = N_-1;
-  Real piNb = pi/Nb;
-  for (int j=0; j<Nb; ++j) {
-    sin_table_[j] = sin(j*piNb);
-    cos_table_[j] = cos(j*piNb);
-  }
-  int rank=1;
-  int n[1];
-  n[0] = Nb;
-  realfft_plan_  = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, flags);
+  // Build an FFTW plan that can be applied to a number of different arrays,
+  // using the FFTW guru interface. See FFTW guru documentation.
+  Real* tmp = (Real*) fftw_malloc(N_*sizeof(Real)); 
+  fftw_r2r_kind fftw_kind = FFTW_REDFT00;
+  cosfftw_plan_  = fftw_plan_r2r_1d(N,tmp,tmp,fftw_kind,flags_);
+  fftw_free(tmp);
 }
 
 ChebyTransform::ChebyTransform(const ChebyTransform& t) 
   : 
   N_(),
-  sin_table_(new Real[N_-1]),
-  cos_table_(new Real[N_-1]),
-  q_(new Real[N_-1]),
-  Q_(new fftw_complex[(N_-1)/2 +1]),
-  flags_(t.flags_),
-  realfft_plan_(0) 
+  flags_(t.flags_ | FFTW_DESTROY_INPUT),
+  cosfftw_plan_(0) 
 {
   assert(N_ >0);
-  assert(N_%2 == 1);
 
-  int Nb = N_-1;
-  Real piNb = pi/Nb;
-  for (int j=0; j<Nb; ++j) {
-    sin_table_[j] = sin(j*piNb);
-    cos_table_[j] = cos(j*piNb);
-  }
-  int rank=1;
-  int n[1];
-  n[0] = Nb;
-  realfft_plan_  = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, flags_);
+  // Build an FFTW plan that can be applied to a number of different arrays,
+  // using the FFTW guru interface. See FFTW guru documentation.
+  Real* tmp = (Real*) fftw_malloc(N_*sizeof(Real)); 
+  fftw_r2r_kind fftw_kind = FFTW_REDFT00;
+  cosfftw_plan_ = fftw_plan_r2r_1d(N_, tmp, tmp, fftw_kind, flags_);
+  fftw_free(tmp);
 }
 
 ChebyTransform& ChebyTransform::operator=(const ChebyTransform& t) {
   if (&t == this)
     return *this;
 
-  delete[] sin_table_;
-  delete[] cos_table_;
-  delete[] q_;
-  delete[] Q_;
-  rfftwnd_destroy_plan(realfft_plan_);
+  fftw_destroy_plan(cosfftw_plan_);
 
   N_ = t.N_;
-  sin_table_ = new Real[N_-1];
-  cos_table_ = new Real[N_-1];
-  q_ = new Real[N_-1];
-  Q_ = new fftw_complex[(N_-1)/2 +1];
   flags_ = t.flags_;
 
   assert(N_ >0);
-  assert(N_%2 == 1);
 
-  int Nb = N_-1;
-  Real piNb = pi/Nb;
-  for (int j=0; j<Nb; ++j) {
-    sin_table_[j] = sin(j*piNb);
-    cos_table_[j] = cos(j*piNb);
-  }
-  int rank=1;
-  int n[1];
-  n[0] = Nb;
-  realfft_plan_ = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, flags_);
+  Real* tmp = (Real*) fftw_malloc(N_*sizeof(Real)); 
+  fftw_r2r_kind fftw_kind = FFTW_REDFT00;
+  cosfftw_plan_ = fftw_plan_r2r_1d(N_, tmp, tmp, fftw_kind,flags_);
+  fftw_free(tmp);
+  
   return *this;
 }
 
 ChebyTransform::~ChebyTransform() {
-  delete[] sin_table_;
-  delete[] cos_table_;
-  delete[] q_;
-  delete[] Q_;
-  rfftwnd_destroy_plan(realfft_plan_);
+  fftw_destroy_plan(cosfftw_plan_);
 }
 
+/***********************************************************************
 void ChebyTransform::save(const ChebyCoeff& u, const string& name, int deriv) const {
   ChebyCoeff tmp(u.length(), u.a(), u.b(), u.state());
   if (deriv == 0) 
@@ -1330,47 +1359,13 @@ void ChebyTransform::save(const ComplexChebyCoeff& u, const string& name, int de
 void ChebyTransform::chebyfft(ChebyCoeff& u) const {
   assert(u.length() == N_);
   assert(u.state() == Physical);
-  //int rank = 1;
-  int Nb = N_-1;
-  int n[1];
-  n[0] = Nb;
+  fftw_execute_r2r(cosfftw_plan_, u.data_, u.data_);
 
-  double c = 2.0/Real(Nb);
-
-  Real a;
-  Real b;
-
-  int j; // MSVC++ FOR-SCOPE BUG
-  for (j=0; j<N_; ++j) 
-    u[j] *= c;
+  Real nrm = 1.0/(N_-1); // needed because FFTW does unnormalized transforms
+  u[0] *= 0.5*nrm;
+  for (int n=1; n<N_; ++n)
+    u[n] *= nrm;
   
-  // Transform data according to NR formula 
-  for (j=0; j<Nb; ++j) {
-    a = u[j];
-    b = u[Nb-j];
-    q_[j] = 0.5*(a+b) - sin_table_[j]*(a-b);
-  }
-
-  rfftwnd_one_real_to_complex(realfft_plan_, q_, Q_);
-
-  // Initialize recursion for unpacking transform
-  Real sum = 0.5*(u[0] - u[Nb]);
-  u[0] = Q_[0].re;
-
-  for (j=1; j<Nb; ++j)
-    sum += u[j]*cos_table_[j];
-  u[1] = sum;
-	
-  for (int k=1; k<Nb/2; ++k) {      // N is even 
-    u[2*k]   = Q_[k].re;
-    u[2*k+1] = (sum -= Q_[k].im); 
-  }
-  u[Nb] = Q_[Nb/2].re;
-
-  // Copy transformed data back into main data array
-
-  // 0th elem is different due to reln between cheby and cos transforms.
-  u[0] *= 0.5; // can fold into initial assigment for efficiency.
   u.setState(Spectral);
   return;
 
@@ -1380,39 +1375,13 @@ void ChebyTransform::ichebyfft(ChebyCoeff& u) const {
   assert(u.length() == N_);
   assert(u.state() == Spectral);
   //int rank =1;
-  int Nb = N_-1;
-  int n[1];
-  n[0] = Nb;
-  Real a;
-  Real b;
 
-  // Copy data spread through memory into a stride-1 scratch array.
   // 0th elem is different due to reln between cheby and cos transforms.
   u[0] *= 2.0;
-	
-  // Transform data according to NR formula 
-  int j; // MSVC++ FOR-SCOPE BUG
-  for (j=0; j<Nb; ++j) {
-    a = u[j];
-    b = u[Nb-j];
-    q_[j] = 0.5*(a+b) - sin_table_[j]*(a-b);
-  }
-
-  rfftwnd_one_real_to_complex(realfft_plan_, q_, Q_);
-
-  // Initialize recursion for unpacking transform
-  Real sum = 0.5*(u[0] - u[Nb]);
-  u[0] = Q_[0].re;
-
-  for (j=1; j<Nb; ++j)
-    sum += u[j]*cos_table_[j];
-  u[1] = sum;
-	
-  for (int k=1; k<Nb/2; ++k) {      // N is even 
-    u[2*k]   = Q_[k].re;
-    u[2*k+1] = (sum -= Q_[k].im); 
-  }
-  u[Nb] = Q_[Nb/2].re;
+  fftw_execute_r2r(cosfftw_plan_, u.data_, u.data_);
+  for (int n=0; n<N_; ++n)
+    u[n] *= 0.5;
+  
   u.setState(Physical);
   return;
 }
@@ -1446,7 +1415,7 @@ void ChebyTransform::makePhysical(ComplexChebyCoeff& u) const {
     ichebyfft(u);
   return;
 }
-
+******************************************************************/
 ostream& operator<<(ostream& os, const ComplexChebyCoeff& u) {
   for (int i=0; i<u.length(); ++i)
     os << '(' << u.re[i] << ", " << u.im[i] << ")\n";
