@@ -192,9 +192,10 @@ void DNS::advance(FlowField& u, FlowField& q, int Nsteps) {
 
   // Error check
   if (!main_algorithm_->full() && !init_algorithm_) {
-    cerr << "DNS::advance(u,q,Nsteps) : the main algorithm is uninitialized,\n" 
-	 << "and the initialization algorithm is not set. This should not be possible.\n"
-	 << "Please submit a bug report (see documentation)." << endl;
+    cerr << "DNS::advance(u,q,Nsteps) : the main algorithm is uninitialized,\n"
+	 << "and the initialization algorithm is not set. This should not be\n"
+	 << "possible. Please submit a bug report (see documentation)." 
+	 << endl;
     exit(1);
   }
   int n=0;
@@ -613,7 +614,7 @@ MultistepDNS::MultistepDNS(FlowField& u, const ChebyCoeff& Ubase,
     u_[0] = u;
     navierstokesNL(u_[0], Ubase_, f_[0], tmp_, flags_.nonlinearity);
 
-    cfl_ = u_[0].CFLfactor();
+    cfl_ = u_[0].CFLfactor(Ubase_);
     cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
   }
 
@@ -771,7 +772,7 @@ void MultistepDNS::advance(FlowField& un, FlowField& qn, int Nsteps) {
       cout << '.' << flush;
   }
   
-  cfl_ = u_[0].CFLfactor();
+  cfl_ = u_[0].CFLfactor(Ubase_);
   cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
     
   // If using dealiasing, set flag in FlowField that compactifies binary IO 
@@ -812,12 +813,13 @@ bool MultistepDNS::push(const FlowField& un) {
   if (order_ > 0) {
     u_[0] = un;
     navierstokesNL(u_[0], Ubase_, f_[0], tmp_, flags_.nonlinearity);
-    cfl_ = u_[0].CFLfactor();
+    cfl_ = u_[0].CFLfactor(Ubase_);
     cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
   }
 
   t_ += dt_;
   --countdown_;
+
   return full();
 }
 
@@ -1098,7 +1100,7 @@ void RungeKuttaDNS::advance(FlowField& un, FlowField& qn, int Nsteps) {
       cout << '.' << flush;
   }
     
-  cfl_ = un.CFLfactor();
+  cfl_ = un.CFLfactor(Ubase_);
   cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
   
   // If using dealiasing, set flag in FlowField that compactifies binary IO 
@@ -1422,7 +1424,7 @@ void CNABstyleDNS::advance(FlowField& un, FlowField& qn, int Nsteps) {
       cout << '.' << flush;
   }
 
-  cfl_ = un.CFLfactor();
+  cfl_ = un.CFLfactor(Ubase_);
   cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
   
   // If using dealiasing, set flag in FlowField that compactifies binary IO 
@@ -1537,7 +1539,7 @@ PPEDNS::PPEDNS(FlowField& u, const ChebyCoeff& Ubase,
     grad(p, tmp);
     f_[0] -= tmp;
 
-    cfl_ = u_[0].CFLfactor();
+    cfl_ = u_[0].CFLfactor(Ubase_);
     cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
   }
 
@@ -1700,7 +1702,7 @@ void PPEDNS::advance(FlowField& un, FlowField& qn, int Nsteps) {
       cout << '.' << flush;
   }
   
-  cfl_ = u_[0].CFLfactor();
+  cfl_ = u_[0].CFLfactor(Ubase_);
   cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
     
   // If using dealiasing, set flag in FlowField that compactifies binary IO 
@@ -1747,7 +1749,7 @@ bool PPEDNS::push(const FlowField& un) {
     grad(p, tmp_);
     f_[0] -= tmp_;
 
-    cfl_ = u_[0].CFLfactor();
+    cfl_ = u_[0].CFLfactor(Ubase_);
     cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
   }
 
