@@ -49,17 +49,12 @@ FlowField::FlowField()
   dealiasIO_(false),
   rdata_(0),
   cdata_(0),
-  q_(0),
-  Q_(0),
   scratch_(0),
-  sin_table_(0),
-  cos_table_(0),
   xzstate_(Spectral),
   ystate_(Spectral),
   xz_plan_(0),
   xz_iplan_(0),
-  y_plan_(0),
-  y_iplan_(0)
+  y_plan_(0)
 {}
 
 FlowField::FlowField(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz, 
@@ -79,17 +74,12 @@ FlowField::FlowField(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz,
   dealiasIO_(false),
   rdata_(0),
   cdata_(0),
-  q_(0),
-  Q_(0),
   scratch_(0),
-  sin_table_(0),
-  cos_table_(0),
   xzstate_(xzstate),
   ystate_(ystate),
   xz_plan_(0),
   xz_iplan_(0),
-  y_plan_(0),
-  y_iplan_(0)
+  y_plan_(0)
 {
   // This isn't in classic C++ initialization style, but it consolidates
   // all resize and initialization code, and the relative overhead is neglgble.
@@ -113,17 +103,12 @@ FlowField::FlowField(int Nx, int Ny, int Nz, int Nd, int tensorOrder, Real Lx,
   dealiasIO_(false),
   rdata_(0),
   cdata_(0),
-  q_(0),
-  Q_(0),
   scratch_(0),
-  sin_table_(0),
-  cos_table_(0),
   xzstate_(xzstate),
   ystate_(ystate),
   xz_plan_(0),
   xz_iplan_(0),
-  y_plan_(0),
-  y_iplan_(0)
+  y_plan_(0)
 {
   // This isn't in classic C++ initialization style, but it consolidates
   // all resize and initialization code, and the relative overhead is neglgble.
@@ -145,18 +130,12 @@ FlowField::FlowField(const FlowField& f)
   dealiasIO_(f.dealiasIO_),
   rdata_(0),
   cdata_(0),
-  q_(0),
-  Q_(0),
   scratch_(0),
-  sin_table_(0),
-  cos_table_(0),
   xzstate_(Spectral),
   ystate_(Spectral),
   xz_plan_(0),
   xz_iplan_(0),
-  y_plan_(0),
-  y_iplan_(0)
-
+  y_plan_(0)
 {
   resize(f.Nx_, f.Ny_, f.Nz_, f.Nd_, f.Lx_, f.Lz_, f.a_, f.b_);
   setState(f.xzstate_, f.ystate_);
@@ -165,7 +144,7 @@ FlowField::FlowField(const FlowField& f)
   for (int i=0; i<N; ++i)
     rdata_[i] = f.rdata_[i];
 
-  fftw_initialize();
+  fftw_initialize(FFTW_ESTIMATE);
 }
 
 FlowField::FlowField(const string& filebase) 
@@ -183,17 +162,12 @@ FlowField::FlowField(const string& filebase)
   dealiasIO_(false),
   rdata_(0),
   cdata_(0),
-  q_(0),
-  Q_(0),
   scratch_(0),
-  sin_table_(0),
-  cos_table_(0),
   xzstate_(Spectral),
   ystate_(Spectral),
   xz_plan_(0),
   xz_iplan_(0),
-  y_plan_(0),
-  y_iplan_(0)
+  y_plan_(0)
 {
   
   string filename(filebase);
@@ -290,17 +264,12 @@ FlowField::FlowField(const string& filebase, const string& chflow_version)
   dealiasIO_(false),
   rdata_(0),
   cdata_(0),
-  q_(0),
-  Q_(0),
   scratch_(0),
-  sin_table_(0),
-  cos_table_(0),
   xzstate_(Spectral),
   ystate_(Spectral),
   xz_plan_(0),
   xz_iplan_(0),
-  y_plan_(0),
-  y_iplan_(0)
+  y_plan_(0)
 {
   
   string filename(filebase);
@@ -411,17 +380,12 @@ FlowField& FlowField::operator=(const FlowField& f) {
 }
 
 FlowField::~FlowField() {
-  delete[] cos_table_;
-  delete[] sin_table_;
-  delete[] Q_;
-  delete[] q_;
-  delete[] scratch_;
-  delete[] rdata_;
+  fftw_free(scratch_);
+  fftw_free(rdata_);
   
-  rfftwnd_destroy_plan(y_plan_);
-  rfftwnd_destroy_plan(y_iplan_);
-  rfftwnd_destroy_plan(xz_plan_);
-  rfftwnd_destroy_plan(xz_iplan_);
+  fftw_destroy_plan(y_plan_);
+  fftw_destroy_plan(xz_plan_);
+  fftw_destroy_plan(xz_iplan_);
 }
 
 bool FlowField::isNull() {
@@ -430,16 +394,13 @@ bool FlowField::isNull() {
 
 void FlowField::resize(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz, 
 		       Real a, Real b, int fftw_flags) {
-  delete[] cos_table_;
-  delete[] sin_table_;
-  delete[] Q_;
-  delete[] q_;
-  delete[] scratch_;
-  delete[] rdata_;
-  if (y_plan_) rfftwnd_destroy_plan(y_plan_);
-  if (y_iplan_) rfftwnd_destroy_plan(y_iplan_);
-  if (xz_plan_) rfftwnd_destroy_plan(xz_plan_);
-  if (xz_iplan_) rfftwnd_destroy_plan(xz_iplan_);
+
+  // INEFFICIENT if geometry doesn't change. Should check.
+  fftw_free(scratch_);
+  fftw_free(rdata_);
+  if (y_plan_) fftw_destroy_plan(y_plan_);
+  if (xz_plan_) fftw_destroy_plan(xz_plan_);
+  if (xz_iplan_) fftw_destroy_plan(xz_iplan_);
   
   Nx_ = Nx;
   Ny_ = Ny;
@@ -460,44 +421,68 @@ void FlowField::resize(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz,
   
   Nzpad_ = 2*(Nz_/2+1);
   Nzpad2_ = Nz_/2+1;
-  rdata_ = new Real[Nx_ * Ny_ * Nzpad_ * Nd_];
-  cdata_ = (Complex*)rdata_;
+  rdata_ = (Real*) fftw_malloc((Nx_*Ny_*Nzpad_*Nd_)*sizeof(Real));
+  cdata_ = (Complex*) rdata_;
     
   int N = Nd_*Ny_*Nx_*Nzpad_;
   for (int i=0; i<N; ++i)
     rdata_[i] = 0.0;
 
-  int Nybar = (Ny_ > 0) ? Ny_-1 : 0;
-  q_ = new Real[Nybar];
-  Q_ = new fftw_complex[Nybar/2 +1];
-  scratch_ = new Real[Ny_];
-  sin_table_ = new Real[Nybar];
-  cos_table_ = new Real[Nybar];
+  scratch_ = (Real*) fftw_malloc(Ny_*sizeof(Real));
 
   fftw_initialize(fftw_flags);
 }
   
-void FlowField::fftw_initialize(int fftw_flags) {
-  int Ny1 = Ny_-1;
-  Real piNy1 = pi/Ny1;
-  for (int j=0; j<Ny1; ++j) {
-    sin_table_[j] = sin(j*piNy1);
-    cos_table_[j] = cos(j*piNy1);
-  }
-  int xz_flags = fftw_flags | FFTW_IN_PLACE;
-  int y_flags = fftw_flags;
+void FlowField::fftw_initialize(int flags) {
+  flags = flags | FFTW_DESTROY_INPUT;
   
   if (Nx_ !=0 && Nz_ !=0) {
-    xz_plan_ = rfftw2d_create_plan(Nx_, Nz_, FFTW_REAL_TO_COMPLEX,  xz_flags);
-    xz_iplan_ = rfftw2d_create_plan(Nx_, Nz_, FFTW_COMPLEX_TO_REAL, xz_flags);
+    fftw_complex* fcdata = (fftw_complex*) cdata_;
+
+    const int howmany = Ny_*Nd_;
+    const int rank = 2;
+
+    // These params describe the structure of the real-valued array 
+    int real_n[rank];
+    real_n[0] = Nx_;
+    real_n[1] = Nz_;
+    int real_embed[rank];
+    real_embed[0] = Nx_;
+    real_embed[1] = Nzpad_;
+    const int real_stride = 1;
+    const int real_dist = Nx_*Nzpad_;
+
+    // These params describe the structure of the complex-valued array 
+    int cplx_n[rank];
+    cplx_n[0] = Nx_;
+    cplx_n[1] = Nzpad2_;
+    int cplx_embed[rank];
+    cplx_embed[0] = Nx_;
+    cplx_embed[1] = Nzpad2_;
+    const int cplx_stride = 1;
+    const int cplx_dist = Nx_*Nzpad2_;
+
+    // Real -> Complex transform parameters
+    xz_plan_ = 
+      fftw_plan_many_dft_r2c(rank, real_n, howmany, 
+			     rdata_, real_embed, real_stride, real_dist, 
+			     fcdata, cplx_embed, cplx_stride, cplx_dist, 
+			     flags);
+
+    xz_iplan_ = 
+      fftw_plan_many_dft_c2r(rank, real_n, howmany, 
+			     fcdata, cplx_embed, cplx_stride, cplx_dist, 
+			     rdata_, real_embed, real_stride, real_dist, 
+			     flags);
   }
-  if (Ny_ != 0) {
-    int rank=1;
-    int n[1];
-    n[0] = Ny1;
-    y_plan_  = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, y_flags);
-    y_iplan_ = rfftwnd_create_plan(rank, n, FFTW_COMPLEX_TO_REAL, y_flags);
+  else {
+    xz_plan_ = 0;
+    xz_iplan_ = 0;
   }
+  if (Ny_ != 0) 
+    y_plan_  = fftw_plan_r2r_1d(Ny_,scratch_,scratch_,FFTW_REDFT00, flags);
+  else 
+   y_plan_  = 0;
 }
 
 void FlowField::rescale(Real Lx, Real Lz) {
@@ -707,22 +692,14 @@ void FlowField::interpolate(const FlowField& U) {
 }
 ****************************************************************/
 
-void FlowField::optimizeFFTW() {
-  rfftwnd_destroy_plan(xz_plan_);
-  rfftwnd_destroy_plan(xz_iplan_);
-  rfftwnd_destroy_plan(y_plan_);
-  rfftwnd_destroy_plan(y_iplan_);
+void FlowField::optimizeFFTW(int flags) {
+  fftw_destroy_plan(xz_plan_);
+  fftw_destroy_plan(xz_iplan_);
+  fftw_destroy_plan(y_plan_);
 
-  int xz_flags = FFTW_MEASURE | FFTW_IN_PLACE | FFTW_USE_WISDOM;
-  int y_flags = FFTW_MEASURE | FFTW_USE_WISDOM;
-  int rank=1;
-  int n[1];
-  n[0] = Ny_-1;
+  flags = flags | FFTW_DESTROY_INPUT;
 
-  xz_plan_ = rfftw2d_create_plan(Nx_, Nz_, FFTW_REAL_TO_COMPLEX,  xz_flags);
-  xz_iplan_ = rfftw2d_create_plan(Nx_, Nz_, FFTW_COMPLEX_TO_REAL, xz_flags);
-  y_plan_  = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, y_flags);
-  y_iplan_ = rfftwnd_create_plan(rank, n, FFTW_COMPLEX_TO_REAL, y_flags);
+  fftw_initialize(flags);
 }
 
 ComplexChebyCoeff FlowField::profile(int mx, int mz, int i) const {
@@ -790,7 +767,7 @@ BasisFunc FlowField::profile(int mx, int mz) const {
 void FlowField::addProfile(const BasisFunc& profile, bool addconj) {
   assert(xzstate_== Spectral);
   assert(ystate_ == profile.state());
-  assert(Nd_ == 3);
+  assert(Nd_ == profile.Nd());
   int m_x = mx(profile.kx());
   int m_z = mz(profile.kz());
   for (int i=0; i<Nd_; ++i)
@@ -922,8 +899,9 @@ FlowField& FlowField::operator *= (const FlowField& U) {
   
 void FlowField::realfft_xz() {
   assert (xzstate_ == Physical);
+  fftw_execute(xz_plan_);
   // args are (plan, howmany, in, istride, idist, ostride, odist)
-  rfftwnd_real_to_complex(xz_plan_, Nd_*Ny_, rdata_, 1, Nx_*Nzpad_, 0,0,0);
+  //rfftwnd_real_to_complex(xz_plan_, Nd_*Ny_, rdata_, 1, Nx_*Nzpad_, 0,0,0);
   int Ntotal = Nd_* Nx_ * Ny_ * Nzpad_;
   Real scale = 1.0/(Nx_*Nz_);
   for (int i=0; i<Ntotal; ++i)
@@ -937,117 +915,55 @@ void FlowField::irealfft_xz() {
   //cout << "howmany = " << Nd_*Ny_ << endl;
   //cout << "idist = " << Nx_*Nzpad_/2 << endl;
 
-  rfftwnd_complex_to_real(xz_iplan_, Nd_*Ny_, (fftw_complex*)rdata_, 1, Nx_*Nzpad_/2, 0,0,0);
+  fftw_execute(xz_iplan_);
+  //rfftwnd_complex_to_real(xz_iplan_, Nd_*Ny_, (fftw_complex*)rdata_, 1, Nx_*Nzpad_/2, 0,0,0);
   xzstate_ = Physical;
 }
 
 void FlowField::chebyfft_y() {
   assert(ystate_ == Physical);
-  //int rank =1;
-  int Ny1 = Ny_-1;
-  int n[1];
-  n[0] = Ny1;
-
-  Real c = 2.0/(Ny_-1);
-
+  Real nrm = 1.0/(Ny_-1); // needed because FFTW does unnormalized transforms
   for (int i=0; i<Nd_; ++i) 
     for (int nx=0; nx<Nx_; ++nx) 
       for (int nz=0; nz<Nzpad_; ++nz) {
-	Real a;
-	Real b;
 
 	// Copy data spread through memory into a stride-1 scratch array.
-	int j; // MSVC++ FOR-SCOPE BUG
-	for (j=0; j<Ny_; ++j) 
-	  scratch_[j] = c*rdata_[flatten(nx, j, nz, i)];
-	
-	// Transform data according to NR formula 
-	for (j=0; j<Ny1; ++j) {
-	  a = scratch_[j];
-	  b = scratch_[Ny1-j];
-	  q_[j] = 0.5*(a+b) - sin_table_[j]*(a-b);
-	}
+	for (int ny=0; ny<Ny_; ++ny) 
+	  scratch_[ny] = rdata_[flatten(nx, ny, nz, i)];
 
-	//rfftwnd_one_real_to_complex(y_plan_, q_, Q_);
-	call_fftw();
+	// Transform data
+	fftw_execute_r2r(y_plan_, scratch_, scratch_);
 
-	// Initialize recursion for unpacking transform
-	Real sum = 0.5*(scratch_[0] - scratch_[Ny1]);
-	scratch_[0] = Q_[0].re;
-
-	for (j=1; j<Ny1; ++j)
-	  sum += scratch_[j]*cos_table_[j];
-	scratch_[1] = sum;
-	
-	for (int k=1; k<Ny1/2; ++k) {      // N is even 
-	  scratch_[2*k]   = Q_[k].re;
-	  scratch_[2*k+1] = (sum -= Q_[k].im); 
-	}
-	scratch_[Ny1] = Q_[Ny1/2].re;
-
-	// Copy transformed data back into main data array
-
-	// 0th elem is different due to reln between cheby and cos transforms.
-	rdata_[flatten(nx, 0, nz, i)] = 0.5*scratch_[0];
-	for (j=1; j<Ny_; ++j) 
-	  rdata_[flatten(nx, j, nz, i)] = scratch_[j];
+	// Copy back to multi-d arrays, normalizing on the way
+	// 0th elem is different because of reln btwn cos and Cheb transforms
+	rdata_[flatten(nx, 0, nz, i)] = 0.5*nrm*scratch_[0]; 
+	for (int ny=1; ny<Ny_; ++ny) 
+	  rdata_[flatten(nx, ny, nz, i)] = nrm*scratch_[ny];
       }
   ystate_ = Spectral;
   return;
 }
 
-void FlowField::call_fftw() {
-  rfftwnd_one_real_to_complex(y_plan_, q_, Q_);
-}  
 void FlowField::ichebyfft_y() {
   assert(ystate_ == Spectral);
-
-  //int rank =1;
-  int Ny1 = Ny_-1;
-  int n[1];
-  n[0] = Ny1;
-
   for (int i=0; i<Nd_; ++i) 
     for (int nx=0; nx<Nx_; ++nx) 
       for (int nz=0; nz<Nzpad_; ++nz) {
-	Real a;
-	Real b;
 
 	// Copy data spread through memory into a stride-1 scratch array.
-	// 0th elem is different due to reln between cheby and cos transforms.
-	scratch_[0] = 2*rdata_[flatten(nx, 0, nz, i)];
-	int j; // MSVC++ FOR-SCOPE BUG
-	for (j=1; j<Ny_; ++j) 
-	  scratch_[j] = rdata_[flatten(nx, j, nz, i)];
+	// 0th elem is different because of reln btwn cos and Cheb transforms
+	scratch_[0] = 2.0*rdata_[flatten(nx, 0, nz, i)];
+	for (int ny=1; ny<Ny_; ++ny) 
+	  scratch_[ny] = rdata_[flatten(nx, ny, nz, i)];
 	
-	// Transform data according to NR formula 
-	for (j=0; j<Ny1; ++j) {
-	  a = scratch_[j];
-	  b = scratch_[Ny1-j];
-	  q_[j] = 0.5*(a+b) - sin_table_[j]*(a-b);
-	}
-
-	//rfftwnd_one_real_to_complex(y_plan_, q_, Q_);
-	call_fftw();
-
-	// Initialize recursion for unpacking transform
-	Real sum = 0.5*(scratch_[0] - scratch_[Ny1]);
-	scratch_[0] = Q_[0].re;
-
-	for (j=1; j<Ny1; ++j)
-	  sum += scratch_[j]*cos_table_[j];
-	scratch_[1] = sum;
-	
-	for (int k=1; k<Ny1/2; ++k) {      // N is even 
-	  scratch_[2*k]   = Q_[k].re;
-	  scratch_[2*k+1] = (sum -= Q_[k].im); 
-	}
-	scratch_[Ny1] = Q_[Ny1/2].re;
+	// Transform data
+	fftw_execute_r2r(y_plan_, scratch_, scratch_);
 
 	// Copy transformed data back into main data array
-	for (j=0; j<Ny_; ++j) 
-	  rdata_[flatten(nx, j, nz, i)] = scratch_[j];
-	}
+	for (int ny=0; ny<Ny_; ++ny) 
+	  rdata_[flatten(nx, ny, nz, i)] = 0.5*scratch_[ny];
+
+      }
   ystate_ = Physical;
   return;
 }
@@ -1682,6 +1598,21 @@ void swap(FlowField& f, FlowField& g) {
 
   f.cdata_ = (Complex*)f.rdata_;
   g.cdata_ = (Complex*)g.rdata_;
+
+  // We must swap the FFTW3 plans, as well, since they're tied to specific 
+  // memory locations. This burned me when I upgraded FFTW2->FFTW3.
+  fftw_plan tmp_plan;
+  tmp_plan = f.xz_plan_;
+  f.xz_plan_ = g.xz_plan_;
+  g.xz_plan_ = tmp_plan;
+
+  tmp_plan = f.xz_iplan_;
+  f.xz_iplan_ = g.xz_iplan_;
+  g.xz_iplan_ = tmp_plan;
+  
+  tmp_plan = f.y_plan_;
+  f.y_plan_ = g.y_plan_;
+  g.y_plan_ = tmp_plan;
 }
   
 
