@@ -29,11 +29,7 @@
 #ifndef FLOWFIELD_H
 #define FLOWFIELD_H
 
-#ifdef WIN32
-#include <rfftw.h>
-#else
-#include <drfftw.h>
-#endif
+#include <fftw3.h>
 #include "mathdefs.h"
 #include "cfvector.h"
 #include "chebyshev.h"
@@ -70,12 +66,12 @@ public:
   FlowField();
   FlowField(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz, Real a, Real b,
 	    fieldstate xzstate=Spectral, fieldstate ystate=Spectral, 
-	    int fftw_flags = FFTW_ESTIMATE | FFTW_USE_WISDOM);
+	    int fftw_flags = FFTW_ESTIMATE);
 
   FlowField(int Nx, int Ny, int Nz, int Nd, int tensorOrder, 
 	    Real Lx, Real Lz, Real a, Real b,
 	    fieldstate xzstate=Spectral, fieldstate ystate=Spectral, 
-	    int fftw_flags = FFTW_ESTIMATE | FFTW_USE_WISDOM);
+	    int fftw_flags = FFTW_ESTIMATE);
 
   FlowField(const FlowField& u);
   FlowField(const string& filebase); // opens filebase.ff
@@ -86,11 +82,11 @@ public:
   
   bool isNull(); // true if Nx=Ny=Nz=Nd=0
   void resize(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz, Real a, Real b,
-	      int fftw_flags = FFTW_ESTIMATE | FFTW_USE_WISDOM);
+	      int fftw_flags = FFTW_ESTIMATE);
 
   void rescale(Real Lx, Real Lz);
   void interpolate(const FlowField& u); // interpolate U onto this grid.
-  void optimizeFFTW();
+  void optimizeFFTW(int fftw_flags = FFTW_MEASURE);
   void call_fftw(); // debugging
   // Real    operator()(nx,ny,nz,i):  0<=nx<Nx, 0<=ny<Ny, 0<=nz<Nz, 0<=i<Nd
   // Complex cmplx(mx,my,mz,i):       0<=mx<Mx, 0<=my<My, 0<=mz<Mz, 0<=i<Nd
@@ -268,27 +264,21 @@ private:
   
   Real* rdata_;    // stored with indices in order d, Ny, Nx, Nz
   Complex* cdata_; // Complex alias for rdata_ (cdata_ = (Complex*) rdata_).
-
-  Real* q_;         // scratch space for y transforms. Q = realfftw(q)
-  fftw_complex* Q_; // q and Q are meaningless names for temp variables.
   Real* scratch_;   // scratch space for y transforms.
 
-  Real* sin_table_; // sin_table[j] = sin(j*pi/(Ny-1));
-  Real* cos_table_; // cos_table[j] = cos(j*pi/(Ny-1));
-
   fieldstate xzstate_;
   fieldstate ystate_;
   
-  rfftwnd_plan xz_plan_;
-  rfftwnd_plan xz_iplan_;
-  rfftwnd_plan   y_plan_;
-  rfftwnd_plan  y_iplan_;
+  fftw_plan xz_plan_;
+  fftw_plan xz_iplan_;
+  fftw_plan   y_plan_;
+  //fftw_plan  y_iplan_;
   
   inline int flatten(int nx, int ny, int nz, int i) const;
   inline int complex_flatten(int mx, int my, int mz, int i) const;
   inline int flatten(int nx, int ny, int nz, int i, int j) const;
   inline int complex_flatten(int mx, int my, int mz, int i, int j) const;
-  void fftw_initialize(int fftw_flags = FFTW_ESTIMATE | FFTW_USE_WISDOM );
+  void fftw_initialize(int fftw_flags = FFTW_ESTIMATE);
 };
 
 // innerProduct(f,g) == Int f g*  dx dy dz   (/(Lx Ly Lz) if normalize==true)
@@ -473,7 +463,7 @@ inline Real FlowField::z(int nz) const {
 
 inline int FlowField::numXmodes() const {return Nx_;} 
 inline int FlowField::numYmodes() const {return Ny_; }
-inline int FlowField::numZmodes() const {return Nzpad2_;}
+inline int FlowField::numZmodes() const {return Nzpad2_;} // Nzpad2 = Nz/2+1
 
 inline int FlowField::numXgridpts() const {return Nx_;} 
 inline int FlowField::numYgridpts() const {return Ny_; }
