@@ -1,13 +1,16 @@
 /* poissonsovler.h: poisson equation solver
  * Channelflow-0.9
  *
- * Copyright (C) 2005  John F. Gibson  
- *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Copyright (C) 2001-2005  John F. Gibson
+ *
+ * John F. Gibson
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ *
+ * gibson@cns.physics.gatech.edu
+ * jfg@member.fsf.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -78,7 +81,9 @@ protected:
 class PressureSolver : public PoissonSolver {
 public:
   PressureSolver();
- 
+  PressureSolver(int Nx, int Ny, int Nz, Real Lx, Real Lz, Real a, Real b, 
+		 const ChebyCoeff& U, Real nu, NonlinearMethod nonl_method); 
+
   PressureSolver(const FlowField& u, Real nu, NonlinearMethod nonl_method); 
 
   PressureSolver(const FlowField& u, const ChebyCoeff& U, Real nu, 
