@@ -12,18 +12,20 @@ public:
   TurbStats(const string& filebase);
   TurbStats(const ChebyCoeff& Ubase, Real nu);
 
-  void addData(FlowField& u, FlowField& tmp);
+  void reset();
+  void addData(FlowField& ubase, FlowField& tmp);
   void msave(const string& filebase, bool wallunits=false) const;
 
-  ChebyCoeff Umean() const;  // average flow
-  ChebyCoeff ubase() const;  // average fluctuation on base flow
+  ChebyCoeff U() const;          // average of total flow
+  ChebyCoeff dUdy() const;       
+  ChebyCoeff Ubase() const;      // base flow (e.g. parabola)
+  ChebyCoeff ubase() const;      // mean fluctuation on base flow
   ChebyCoeff uu() const;
   ChebyCoeff uv() const;
   ChebyCoeff uw() const;
   ChebyCoeff vv() const;
   ChebyCoeff vw() const;
   ChebyCoeff ww() const;
-  ChebyCoeff dUdy() const;
 
   // wall unit stuff
   Real ustar() const;   
@@ -36,8 +38,8 @@ private:
   int count_;
   Real nu_;          // All ChebyCoeff quantities are sums for means, in utot
   ChebyCoeff Ubase_; // base flow (parabolas, etc).
-  ChebyCoeff Umean_; // mean flow = avg(utot)
   ChebyCoeff ubase_; // mean fluc above base flow: utot = Ubase + ubase
+  ChebyCoeff U_;     // mean flow = avg(utot)
   ChebyCoeff uu_;    // sum_1^count utot utot
   ChebyCoeff uv_;
   ChebyCoeff uw_;
