@@ -12,12 +12,12 @@ int main() {
 
   Real decay = 0.8;
   int N0=65;
-  int N1=33;
+  int N1=65;
 
   Real a0 = -1;
   Real b0 = 1;
 
-  Real a1 = 0;
+  Real a1 = -1;
   Real b1 = 1;
 
   ChebyTransform trans0(N0);
@@ -28,8 +28,8 @@ int main() {
   y0.save("y0");
   y1.save("y1");
   
-  ChebyCoeff f(N0, a0, b0, Physical);
-  ChebyCoeff g(N1, a1, b1, Physical);
+  ChebyCoeff f(N0, a0, b0, Spectral);
+  ChebyCoeff g(N1, a1, b1, Spectral);
   
   f.randomize(decay);
 
@@ -41,30 +41,26 @@ int main() {
   f.save("f");
   g.save("g");
 
-  f.makeSpectral(trans0);
-  g.makeSpectral(trans1);
-  f.save("fc");
-  g.save("gc");
+  f.save("fc", Spectral);
+  g.save("gc", Spectral);
 
-  // Now evaluate/interpolate f and g on uniformly spaced points
+  // Now evaluate/interpolate f and g at the same, uniformly spaced, points
   int M=30;
   Vector f_unif(M);
   Vector g_unif(M);
-  Vector x0(M);
-  Vector x1(M);
-  Real dx0 = (b0-a0)/(M-1);
-  Real dx1 = (b1-a1)/(M-1);
-
+  Vector y2(M);
+  Real dy2 = (b1-a1)/(M-1);
   for (int m=0; m<M; ++m) {
-    x0(m) = a0 +m*dx0;
-    f_unif(m) = f.eval(x0(m));
-    x1(m) = a1 +m*dx1;
-    g_unif(m) = g.eval(x1(m));
+    y2(m) = a1 +m*dy2;
+    f_unif(m) = f.eval(y2(m));
+    g_unif(m) = g.eval(y2(m));
   }
+  Vector e_unif = f_unif - g_unif;
+
+  e_unif.save("e_unif");
   f_unif.save("f_unif");
   g_unif.save("g_unif");
-  x0.save("x0");
-  x1.save("x1");
-  
 
+
+  y2.save("y2");
 }
