@@ -25,7 +25,7 @@ int main() {
     //f[n] = 1-square(yn); //sin(yn);
     //fy[n] = -2*yn; // -2*cos(yn);
     //fyy[n] = -2; // -sin(yn);
-    f[n] = sin(yn);
+    f[n] = 1+sin(yn);
     fy[n] = cos(yn);
     fyy[n] = -sin(yn);
   }
@@ -33,7 +33,11 @@ int main() {
   ChebyTransform t(N);
 
   ChebyCoeff g(f);
-  t.chebyfft(g);
+  g.chebyfft(t);
+  g.ichebyfft(t);
+  if (verbose)
+    cout << "L1Dist(f,g) == " << L1Dist(f,g) << endl;
+  g.chebyfft(t);
   
   ChebyCoeff gy;
   diff(g, gy);
@@ -55,12 +59,12 @@ int main() {
   if (verbose)
     cout << "L1Dist(integrate(diff(gy)), g) == " << err  << endl;
 
-  t.ichebyfft(g);
-  t.ichebyfft(gy);
-  t.ichebyfft(gyy);
-  t.ichebyfft(Ig);
-  t.ichebyfft(Igy);
-  t.ichebyfft(DIg);
+  g.ichebyfft(t);
+  gy.ichebyfft(t);
+  gyy.ichebyfft(t);
+  Ig.ichebyfft(t);
+  Igy.ichebyfft(t);
+  DIg.ichebyfft(t);
 
   err = L1Norm(f-g);
   error += err;
@@ -94,7 +98,7 @@ int main() {
   h.save("h");
 
   ComplexChebyCoeff h2("h");
-  err = L2Dist(h,h2);
+  err = L1Dist(h,h2);
   error += err;
 
   if (verbose)
@@ -108,33 +112,33 @@ int main() {
     h.setState(Spectral);
 
     h.set(0, 1.0 + 0.0*I);
-    t.ichebyfft(h);
+    h.ichebyfft(t);
     h.save("T0");
-    t.chebyfft(h);
+    h.chebyfft(t);
     h.set(0, zero);
 
     h.set(1, 1.0 + 0.0*I);
-    t.ichebyfft(h);
+    h.ichebyfft(t);
     h.save("T1");
-    t.chebyfft(h);
+    h.chebyfft(t);
     h.set(1, zero);
 
     h.set(2, 1.0 + 0.0*I);
-    t.ichebyfft(h);
+    h.ichebyfft(t);
     h.save("T2");
-    t.chebyfft(h);
+    h.chebyfft(t);
     h.set(1, zero);
 
     h.set(2, 1.0 + 0.0*I);
-    t.ichebyfft(h);
+    h.ichebyfft(t);
     h.save("T2");
-    t.chebyfft(h);
+    h.chebyfft(t);
     h.set(2, zero);
 
     h.set(3, 1.0 + 0.0*I);
-    t.ichebyfft(h);
+    h.ichebyfft(t);
     h.save("T3");
-    t.chebyfft(h);
+    h.chebyfft(t);
     h.set(3, zero);
   }
   
