@@ -8,18 +8,19 @@
 const Real EPSILON=1e-12;
 
 void assignComp(FlowField& u, int kx, int kz) {
-  u.setState(Physical, Spectral);
+  u.setState(Physical, Physical);
   u.setToZero();
 
   Real cx = 2*pi*kx/u.Lx();
   Real cz = 2*pi*kz/u.Lz();
-  Real dx = u.Lx()/u.numXgridpts();
-  Real dz = u.Lz()/u.numZgridpts();
-  for (int nx=0; nx<u.numXgridpts(); ++nx) 
-    for (int nz=0; nz<u.numZgridpts(); ++nz) 
-      u(nx, 0, nz, 0) = cos(cx*nx*dx) * cos(cz*nz*dz);
+  Real dx = u.Lx()/u.Nx();
+  Real dz = u.Lz()/u.Nz();
+  for (int ny=0; ny<u.Ny(); ++ny) 
+    for (int nx=0; nx<u.Nx(); ++nx) 
+      for (int nz=0; nz<u.Nz(); ++nz) 
+	u(nx, ny, nz, 0) = cos(cx*nx*dx) * cos(cz*nz*dz);
   
-  u.realfft_xz();
+  u.makeSpectral();
 }
 
 // Set a flowfield to cos(2 pi kx x/Lx) cos(2 pi kz z/Lz) and then
@@ -27,9 +28,9 @@ void assignComp(FlowField& u, int kx, int kz) {
 // are laid out.
 int main(int argc, char* argv[]) {
 
-  const int Nx=8;
-  const int Ny=3;
-  const int Nz=8;
+  const int Nx=32;
+  const int Ny=17;
+  const int Nz=32;
   const int Nd=1;
   const Real Lx=2*pi;
   const Real Lz=2*pi;
@@ -46,7 +47,7 @@ int main(int argc, char* argv[]) {
     
   FlowField u(Nx,Ny,Nz,Nd,Lx,Lz,a,b);
   assignComp(u, kx, kz);
-  u.saveSpectrum("uspec", 0, 0);
+  u.saveSpectrum("uspec", false);
   
   
 }
