@@ -18,7 +18,7 @@ ChebyCoeff interpolate(const ChebyCoeff& u, int N) {
     uinterp[i] = u.eval(x[i]);
 
   ChebyTransform t(N);
-  t.makeSpectral(uinterp);
+  uinterp.makeSpectral(t);
   return uinterp;
 }
   
@@ -85,8 +85,8 @@ int main() {
     Real vas = vs.eval_a();
     Real vbs = vs.eval_b();
 
-    trans.save(v, "v");
-    trans.save(vs, "vs");
+    v.save("v");
+    vs.save("vs");
 
     Real errL1 = L1Dist(v,vs)/L1Norm(v);
     Real errTau = tauDist(v,vs)/L1Norm(v);
